package DataPreprocessing;

import com.opencsv.CSVReader;
import com.opencsv.CSVWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Calendar;
import weka.core.Instances;
import weka.core.converters.ArffSaver;
import weka.core.converters.CSVLoader;

public class DataPreprocessing {
    /*Parametrización de la diferencia entre la latitud minima y maxima y longitud minima y maxima.
     Tamaño de la celda. Creación de los objetos necesarios para el calculo de atributos*/
    private static final int differenceMaxMinLongitude = 140;
    private static final int differenceMaxMinLatitude = 84;
    private static final int divisionCell = 4;
    private static final int maxIndexLatitude = differenceMaxMinLatitude/divisionCell;
    private static final int maxIndexLongitude = differenceMaxMinLongitude/divisionCell;
    private static CalculateAttributes saveAttributes = new CalculateAttributes();
    private static CalculateFlushCar   flushCar = new CalculateFlushCar();
    
    public static void createCell() throws IOException{
        //Creamos el conjunto de celdas
        String [] head = new String[5];
        head[0] = "Date";
        head[1] = "Longitude";
        head[2] = "Latitude";
        head[3] = "DPreviousPosition";
        head[4] = "ID_TAXI";
        for(int i = 0; i <= maxIndexLatitude; i++){
            for(int j = 0; j <= maxIndexLongitude; j++){
                String csv = "C:\\Users\\adriano\\Documents\\NetBeansProjects\\DataPreprocessing\\Cell\\" + j + "," + i + ".csv";
                CSVWriter writer = new CSVWriter(new FileWriter(csv,false));
                writer.writeNext(head);
                writer.close();
           }
       }
    }
     public static void createEndFile() throws IOException{
        //Creamos el fichero al cual se va aplicar posteriormente minería de datos
        String [] head = new String[9];
        head[0] = "Date";
        head[1] = "Latitude";
        head[2] = "Longitude";
        head[3] = "MDPreviousPosition";
        head[4] = "MediumDistance";
        head[5] = "InflowCar";
        head[6] = "OutflowCar";
        head[7] = "TotalInstancesThisHour";
        head[8] = "Class";
        String csv = "C:\\Users\\adriano\\Documents\\NetBeansProjects\\DataPreprocessing\\BD\\DBTrain.csv";
        CSVWriter writer = new CSVWriter(new FileWriter(csv,false));
        writer.writeNext(head);
        writer.close();
    }
    public static void calculateAtributteLatitudeLongitudeTime() throws Exception {
       /*Distribución de cada posicion GPS a su celda correspondiente*/
       String csvFilename = "C:\\Users\\adriano\\Documents\\NetBeansProjects\\DataPreprocessing\\BD\\trainModify.csv";
       CSVReader csvReader = new CSVReader(new FileReader(csvFilename));
       int contador = 0;
       String [] row = null;
       String [] information;
       String [] result;
       csvReader.readNext();
       while((row = csvReader.readNext()) != null){ 
            result = modifyTrackGPS(row[2]);
            String timeStamp  = row[0];
            for(int i = 1; i < result.length;i++){
                double latitude = obtainLatitude(result[i]);
                double longitude = obtainLongitude(result[i]);
                double previousLatitude = obtainLatitude(result[i-1]);
                double previousLongitude = obtainLongitude(result[i-1]);
                double euclideanDistancePreviousPosition = Math.sqrt(Math.pow(latitude-previousLatitude, 2) + Math.pow(longitude-previousLongitude, 2));
                int conversionTimeStamp = (Integer.parseInt(timeStamp) + i*15);
                String date = date(conversionTimeStamp);
                int indexLongitude = (int)((longitude - (-8.684)) / (divisionCell*Math.pow(10,-3)));
                int indexLatitude = (int)((latitude-(41.152)) / (divisionCell*Math.pow(10,-3)));
                if(indexLongitude >= 0 && indexLongitude <= 35 && indexLatitude >= 0 && indexLatitude <= 21){
                    String csv = "C:\\Users\\adriano\\Documents\\NetBeansProjects\\DataPreprocessing\\Cell\\" + indexLongitude + "," + indexLatitude + ".csv";
                    CSVWriter writer = new CSVWriter(new FileWriter(csv,true));
                    information = saveRow(date,longitude,latitude,euclideanDistancePreviousPosition,row[1]);
                    writer.writeNext(information);
                    writer.close();
                }
            }
            contador++;
            System.out.println("Imprimir contador " + contador);
       } 
       csvReader.close();
    }
    public static void calculateAttributesEndFile() throws FileNotFoundException, IOException{
        /*Calcular atributos para el fichero al cual posteriormente le aplicaremos minería de datos*/
        CSVReader csvReader;
        String [] readerRow = null;
        int [] dateAndHour = new int[2];
        File endFile = new File("C:\\Users\\adriano\\Documents\\NetBeansProjects\\DataPreprocessing\\BD\\DBTrain.csv");
        CSVWriter endFileWriter = new CSVWriter(new FileWriter(endFile,true));
        CSVReader csvReaderTest;
        for(int i = 0; i <= maxIndexLatitude; i++){
            for(int j = 0; j <= maxIndexLongitude; j++){
                String read = "C:\\Users\\adriano\\Documents\\NetBeansProjects\\DataPreprocessing\\Cell\\" + j + "," + i + ".csv";
                System.out.println("i j " + i + " " + j );
                csvReader = new CSVReader(new FileReader(read));
                csvReader.readNext();
                while ((readerRow = csvReader.readNext()) != null ){ 
                    dateAndHour = dateAndHour(readerRow[0]);
                    saveAttributes.saveOneInstanceInHour(dateAndHour[0], dateAndHour[1]);
                    saveAttributes.saveDPreviousPosition(dateAndHour[0], dateAndHour[1], Double.parseDouble(readerRow[3]));
                    saveAttributes.saveLatitudeAndLongitude(dateAndHour[0], dateAndHour[1], Double.parseDouble(readerRow[2]), Double.parseDouble(readerRow[1]));
                    flushCar.saveIDTaxis(dateAndHour[0], dateAndHour[1], Integer.parseInt(readerRow[4]));
                }
                csvReader.close();
                flushCar.outFlowCarAndNumTaxis();
                flushCar.sort();
                saveInstanceInEndFile(endFileWriter,i,j);
                saveAttributes.restart();
                flushCar.restart();
            }
        }
        endFileWriter.close();
    }
    public static void saveInstanceInEndFile(CSVWriter csvWriter, int latitude,int longitude){
        /*Escribir una fila en el fichero final*/
        String [] information;
        int countDate = 0;
        for(int i = 0; i<7;i++){
            for(int j = 0; j<24;j++){
                String typeClass =  "Free";
                int numberTaxisNextHour = flushCar.numberOfTaxisNextHour(i, j);
                int numberTaxisCurrentHour = flushCar.numberOfTaxisCurrentHour(i, j);
                typeClass = typeClass(numberTaxisNextHour,numberTaxisCurrentHour);
                information = saveRowEndFile(countDate,i,j ,longitude,latitude,typeClass);
                countDate++;
                if(information[0] != null)
                    csvWriter.writeNext(information);
            }
        }
    }
    public static String [] modifyTrackGPS(String track){
       /*Modificar ristra que contiene todo el trayecto de una taxis a array de ristra donde cada
        posicion se almacena una subristra que contiene una longitud,latitud*/
        String result = "";
        for(int i = 1; i<track.length()-1;i++){
            if(track.charAt(i) == ']' && track.charAt(i+1) == ','){
                result += track.charAt(i);
                result += " ";
                i++;
            }else{
                result += track.charAt(i);
            }
        }
        String [] resultado = result.split(" ");
        return resultado;
    }
    public static double obtainLongitude(String gps){
        /*Obtener longitud de la ristra que contiene longitud,latitud*/
        String longitude = gps.substring(1,gps.indexOf(","));
        return Double.parseDouble(longitude); 
    }
    public static double obtainLatitude(String gps){
        /*Obtener latitud de la ristra que contiene longitud,latitud*/
        String latitude = gps.substring(gps.indexOf(",")+1,gps.indexOf("]"));
        return Double.parseDouble(latitude); 
    }
    public static int[] dateAndHour(String date){
        /*Obtener dia y hora de una fecha*/
        int [] dateAndHour = new int[2];
        int posEndDate = date.indexOf(" ");
        String dateEnd = date.substring(0,posEndDate);
        dateAndHour[0] = Integer.parseInt(dateEnd);
        int posEndHour = date.indexOf(":");
        String hour = date.substring(posEndDate+1,posEndHour);
        dateAndHour[1] = Integer.parseInt(hour);
        return dateAndHour;
    }
    public static String date (int timeStamp){
       /*Conversion TIMESTAMP A formato "fecha hora"*/
       Calendar date = Calendar.getInstance();
       date.setTimeInMillis((long)timeStamp*1000);
       String entryDate = ((date.get(Calendar.DAY_OF_WEEK)-1)+ " " +date.get(Calendar.HOUR_OF_DAY) + ":" + "00");
       return entryDate;
    }
    public static String [] saveRow(String date,double longitude, double latitude, double euclideanDistancePreviousPosition, String idTaxis ){
        /*Escribir una fila en un fichero que representa una celda*/
        String [] information = new String [5];
        information[0] = date;
        information[1] = Double.toString(longitude);
        information[2] = Double.toString(latitude);
        information[3] = Double.toString(Math.rint(euclideanDistancePreviousPosition*1000)/1000);
        information[4] = idTaxis;
        return information;
    }
    public static String [] saveRowEndFile(int date,int i, int j ,int longitude, int latitude, String typeClass){
        /*Escribir en el fichero final una instancia concreta*/
        String [] information = new String[9];
        information[0] = Integer.toString(date);
        information[1] = Integer.toString(latitude);
        information[2] = Integer.toString(longitude);
        int numInstance = saveAttributes.getNumberOfInstancesPerHour(i, j);
        if(numInstance == 0){
            information[3] = "0";
            information[4] = "0";
            information[5] = "0";
            information[6] = "0";
            information[7] = "0";
            information[8] = "Free";
        }else{
            information[3] = Double.toString(Math.rint((saveAttributes.getDPreviousPosition(i, j)/numInstance)*1000)/1000);
            information[4] = Double.toString(Math.rint(saveAttributes.mediumDistance(i,j)*1000)/1000);
            int numCar = flushCar.inflowCar(i, j);
            information[5] = Integer.toString(numCar);
            numCar = flushCar.outflowCar(i, j);
            information[6] = Integer.toString(numCar);
            information[7] = Integer.toString(numInstance);
            information[8] = typeClass;
        }
        return information;
    }
    public static String typeClass(int numberOfTaxisNextHour,int numberOfTaxisCurrentHour){
        /*Etiquetar una instancia*/
        double criticN = flushCar.calculateCriticN();
        double free = criticN*0.25;
        double synchronizedTraffic = criticN*0.90;
        double congested = criticN;
        if(numberOfTaxisCurrentHour <= free /*|| (numberOfTaxisNextHour == 0 && numberOfTaxisCurrentHour != 0)*/){
            return "Free";
        }else if(numberOfTaxisCurrentHour > free && numberOfTaxisCurrentHour <= synchronizedTraffic ){
            return "Synchronized";
        }else{
            return "Congested";
        }
    }
    public static void calculateLatAndLonMinMax() throws FileNotFoundException, IOException{
        /*Calcular latitud y longitud mínima y maxima del dataset*/
        double latitudeMin = 1000;
        double latitudeMax = 0;
        double longitudeMin = 0;
        double longitudeMax = -500;
        String [] row = null;
        int contador = 0;
        String csvFilename = "C:\\Users\\adriano\\Documents\\NetBeansProjects\\DataPreprocessing\\BD\\trainModify.csv";
        CSVReader csvReader = new CSVReader(new FileReader(csvFilename));
        String [] result;
        csvReader.readNext();
        while((row = csvReader.readNext()) != null ){ 
            result = modifyTrackGPS(row[2]);
            for(int i = 1; i < result.length;i++){
                double latitude = obtainLatitude(result[i]);
                double longitude = obtainLongitude(result[i]);
                if(longitude < longitudeMin){
                    longitudeMin = longitude;
                }
                if(longitude > longitudeMax){
                    longitudeMax = longitude;
                }
                if(latitude < latitudeMin){
                    latitudeMin = latitude;
                }
                if(latitude > latitudeMax){
                    latitudeMax = latitude;
                }
                
            }
            System.out.println(contador);
            contador++;
        }
        System.out.println("Longitud minima " + longitudeMin);
        System.out.println("Longitud maxima " + longitudeMax);
        System.out.println("Latitud minima " + latitudeMin);
        System.out.println("Latitude maxima " + latitudeMax);
    }
   
}
