package Interface;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.border.Border;
import javax.swing.border.LineBorder;
import weka.classifiers.Classifier;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.converters.ArffLoader;

/*Clase que contiene todas las funciones de la pestaña que predice el estado actual del flujo de trafico de oporto*/
public class VisualizeFlowTrafficOporto extends javax.swing.JPanel{
    
    private final Classifier clsFutureMapOporto;
    private Instances instances;
    private Instance  instance;
    List <JButton> buttons;
    List <JButton> buttonsFutureMapOporto;
    private static VisualizeMap map;
    
    
    public VisualizeFlowTrafficOporto() throws Exception {
        map = new VisualizeMap();
        this.clsFutureMapOporto = (Classifier)weka.core.SerializationHelper.read(DirectoryModelFutureMapOporto());
        buttons = new ArrayList<>();
        initComponents();
        cellSet();
        loadDataset();
        visualizeMap.add(map);
        
    }
    /*Rellenar el panel con todos los botones que representan las celdas en las que hemos dividido el mapa de Oporto*/
    public void cellSet(){
        Border border = new LineBorder(Color.BLACK, 1);
        double longitudeMin = -8.684;
        double latitudeMin = 41.236;
        double latitudeMax = 41.240;
        double longitudeMax = -8.680;
        JButton button;
        for(int i = 21; i >= 0;i--){
            for(int j = 0; j < 36; j++){
                button = new JButton();
                button.addActionListener(new ActionListenerImpl());
                button.setBorder(border);
                button.setToolTipText("<html> Latitud Mínima: " + Double.toString(latitudeMin) + "<br/>" + " Latitud Maxima: " + Double.toString(latitudeMax) + "<br/>" + "Longitud Mínima: " + Double.toString(longitudeMin) +  "<br/>" + "Longitud Máxima: " +  Double.toString(longitudeMax) + " </html>");
                buttons.add(button);
                panel.add(button);
                longitudeMin += 0.004;
                longitudeMax += 0.004;
                longitudeMin = Math.rint(longitudeMin*1000)/1000;
                longitudeMax = Math.rint(longitudeMax*1000)/1000;
                button.setActionCommand(Integer.toString(i) + "," + Integer.toString(j));
            }
            latitudeMin -= 0.004;
            latitudeMin = Math.rint(latitudeMin*1000)/1000;
            latitudeMax -= 0.004;
            latitudeMax = Math.rint(latitudeMax*1000)/1000;
            longitudeMin = -8.684;
            longitudeMax = -8.680;
        }
    }
    /* Cargar el fichero con el que se realizó la mineria de datos*/
    public void loadDataset() throws IOException{
        ArffLoader loader = new ArffLoader();
        loader.setSource(new File(DirectoryDatabase()));
        instances = loader.getDataSet();
        instances.setClassIndex(instances.numAttributes()-1);
    }
    /* Convertir fecha "dia hora" al indice correspondiente que utilizamos en el fichero de minería de datos para representarlo */
    public int getDate(String date){
        int posEndDayDate = date.indexOf(" ");
        if(posEndDayDate == -1){
            return -1;
        }
        String Daydate = date.substring(0,posEndDayDate);
        int hour = Integer.parseInt(date.substring(posEndDayDate+1));
        if(hour < 0 || hour > 24){
            return -1;
        }
        if(Daydate.equals("Domingo") || Daydate.toLowerCase().equals("domingo") || Daydate.toUpperCase().equals("DOMINGO")){
            return hour;
        }
        if(Daydate.equals("Lunes") || Daydate.toLowerCase().equals("lunes") || Daydate.toUpperCase().equals("LUNES")){
            return 24*1+hour;
        }
        if(Daydate.equals("Martes") || Daydate.toLowerCase().equals("martes") || Daydate.toUpperCase().equals("MARTES")){
            return 24*2+hour;
        }
        if(Daydate.equals("Miercoles") || Daydate.toLowerCase().equals("miercoles") || Daydate.toUpperCase().equals("MIERCOLES")){
            return 24*3+hour;
        }
        if(Daydate.equals("Jueves") || Daydate.toLowerCase().equals("jueves") || Daydate.toUpperCase().equals("JUEVES")){
            return 24*4+hour;
        }
        if(Daydate.equals("Viernes") || Daydate.toLowerCase().equals("viernes") || Daydate.toUpperCase().equals("VIERNES")){
            return 24*5+hour;
        }
        if(Daydate.equals("Sabado") || Daydate.toLowerCase().equals("sabado") || Daydate.toUpperCase().equals("SABADO")){
            return 24*6+hour;
        }
        return -1;
    }  
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        labelDate = new javax.swing.JLabel();
        fieldDate = new javax.swing.JTextField();
        panel = new javax.swing.JPanel();
        buttonStart = new javax.swing.JButton();
        panelLegend = new javax.swing.JPanel();
        labelFree = new javax.swing.JLabel();
        labelSyncrhonized = new javax.swing.JLabel();
        labelCongested = new javax.swing.JLabel();
        buttonSyncrhonized = new javax.swing.JButton();
        buttonCongested = new javax.swing.JButton();
        buttonFree = new javax.swing.JButton();
        visualizeMap = new javax.swing.JPanel();

        setPreferredSize(new java.awt.Dimension(1200, 800));

        labelDate.setText("Fecha:");

        panel.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        panel.setToolTipText("");
        panel.setPreferredSize(new java.awt.Dimension(650, 450));
        panel.setLayout(new java.awt.GridLayout(22, 36));

        buttonStart.setText("Start");
        buttonStart.setToolTipText("");
        buttonStart.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                buttonStartMouseClicked(evt);
            }
        });

        panelLegend.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)), "Leyenda"));

        labelFree.setText("Libre");

        labelSyncrhonized.setText("Sincronizado");

        labelCongested.setText("Congestionado");

        buttonSyncrhonized.setBackground(new java.awt.Color(255, 153, 0));

        buttonCongested.setBackground(new java.awt.Color(255, 0, 0));
        buttonCongested.setText(" ");

        buttonFree.setBackground(new java.awt.Color(51, 255, 0));
        buttonFree.setPreferredSize(new java.awt.Dimension(33, 23));

        javax.swing.GroupLayout panelLegendLayout = new javax.swing.GroupLayout(panelLegend);
        panelLegend.setLayout(panelLegendLayout);
        panelLegendLayout.setHorizontalGroup(
            panelLegendLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelLegendLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelLegendLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(panelLegendLayout.createSequentialGroup()
                        .addComponent(labelFree, javax.swing.GroupLayout.PREFERRED_SIZE, 68, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(40, 40, 40)
                        .addComponent(buttonFree, javax.swing.GroupLayout.DEFAULT_SIZE, 77, Short.MAX_VALUE))
                    .addGroup(panelLegendLayout.createSequentialGroup()
                        .addComponent(labelSyncrhonized)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(buttonSyncrhonized, javax.swing.GroupLayout.PREFERRED_SIZE, 77, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(panelLegendLayout.createSequentialGroup()
                        .addComponent(labelCongested)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(buttonCongested, javax.swing.GroupLayout.PREFERRED_SIZE, 77, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(62, Short.MAX_VALUE))
        );
        panelLegendLayout.setVerticalGroup(
            panelLegendLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(panelLegendLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(panelLegendLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(labelFree, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(buttonFree, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(panelLegendLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(labelSyncrhonized)
                    .addComponent(buttonSyncrhonized, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(25, 25, 25)
                .addGroup(panelLegendLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(labelCongested)
                    .addComponent(buttonCongested, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(21, Short.MAX_VALUE))
        );

        visualizeMap.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)), "Visualización mapa", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Times New Roman", 0, 12))); // NOI18N

        javax.swing.GroupLayout visualizeMapLayout = new javax.swing.GroupLayout(visualizeMap);
        visualizeMap.setLayout(visualizeMapLayout);
        visualizeMapLayout.setHorizontalGroup(
            visualizeMapLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 0, Short.MAX_VALUE)
        );
        visualizeMapLayout.setVerticalGroup(
            visualizeMapLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 366, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addGap(19, 19, 19)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(labelDate, javax.swing.GroupLayout.DEFAULT_SIZE, 60, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(fieldDate, javax.swing.GroupLayout.PREFERRED_SIZE, 122, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(buttonStart))
                    .addComponent(panelLegend, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(visualizeMap, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addGap(30, 30, 30)
                .addComponent(panel, javax.swing.GroupLayout.PREFERRED_SIZE, 849, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(35, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(43, 43, 43)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(panel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.PREFERRED_SIZE, 700, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(fieldDate, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(labelDate)
                            .addComponent(buttonStart))
                        .addGap(40, 40, 40)
                        .addComponent(panelLegend, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(visualizeMap, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(57, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    /*Predecir el flujo de trafico tanto actual como futuro para una hora de un dia concreto cuando cliqueamos el botón start*/
    private void buttonStartMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_buttonStartMouseClicked
        buttonsFutureMapOporto = VisualizeFutureFlowTrafficOporto.getButtons();
        int countButton = 0;
        int dateStateCurrent = 0;
        if(fieldDate.getText().isEmpty() == false){
            int date = getDate(fieldDate.getText());
            if(date == -1){
               JOptionPane.showMessageDialog(panel, "Fecha no correcta","Error",JOptionPane.ERROR_MESSAGE);
               return;
            }
            for(int i = 21; i>=0;i--){
                for(int j = 0; j<36;j++){
                    try {
                        instance = instances.instance(date + 168*36*i + 168*j);
                        double classFutureMapOporto = clsFutureMapOporto.classifyInstance(instance);
                        if(date == 0){
                            dateStateCurrent = 167;
                        }else{
                            dateStateCurrent = date-1;
                        }
                        instance = instances.instance(dateStateCurrent + 168*36*i + 168*j);
                        double classMapOporto= clsFutureMapOporto.classifyInstance(instance);
                        paintMaps(classMapOporto,classFutureMapOporto,countButton);
                        countButton++;
                    } catch (Exception ex) {
                        Logger.getLogger(VisualizeFlowTrafficOporto.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }
            }
        }else{
          JOptionPane.showMessageDialog(panel, "Campo fecha vacío","Error",JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_buttonStartMouseClicked
    /*Pintar una celda con el correspondiente color dependiendo de como se encuentra el tráfico para esa zona a una día y hora concreta.*/
    public void paintMaps(double classMapOporto, double classFutureMapOporto, int indexButton){
        if(classMapOporto == 0){
            buttons.get(indexButton).setBackground(new java.awt.Color(255,153,0));
        }else if (classMapOporto == 1){
            buttons.get(indexButton).setBackground(new java.awt.Color(255,0,0));
        }else if (classMapOporto == 2){
            buttons.get(indexButton).setBackground(new java.awt.Color(51,255,0));
        }
        if(classFutureMapOporto == 0){
            buttonsFutureMapOporto.get(indexButton).setBackground(new java.awt.Color(255,153,0));
        }else if (classFutureMapOporto == 1){
            buttonsFutureMapOporto.get(indexButton).setBackground(new java.awt.Color(255,0,0));
        }else if (classFutureMapOporto == 2){
            buttonsFutureMapOporto.get(indexButton).setBackground(new java.awt.Color(51,255,0));
        }
    }
     /*Obtener directorio donde está almacenado el modelo que predice el estado futuro del flujo de tráfico*/
    public String DirectoryModelFutureMapOporto(){
        File dir = new File("futureState.model");
        return dir.getAbsolutePath();
    }
     /*Obtener directorio donde está almacenado el fichero al que se aplico minería de datos*/
    public String DirectoryDatabase(){
        File dir = new File("DBFinal.arff");
        return dir.getAbsolutePath();
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton buttonCongested;
    private javax.swing.JButton buttonFree;
    private javax.swing.JButton buttonStart;
    private javax.swing.JButton buttonSyncrhonized;
    private javax.swing.JTextField fieldDate;
    private javax.swing.JLabel labelCongested;
    private javax.swing.JLabel labelDate;
    private javax.swing.JLabel labelFree;
    private javax.swing.JLabel labelSyncrhonized;
    private javax.swing.JPanel panel;
    private javax.swing.JPanel panelLegend;
    private javax.swing.JPanel visualizeMap;
    // End of variables declaration//GEN-END:variables

    private static class ActionListenerImpl implements ActionListener {

        public ActionListenerImpl() {
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            int posEndLatitude = e.getActionCommand().indexOf(",");
            double indexLatitude = Integer.parseInt(e.getActionCommand().substring(0,posEndLatitude));
            double latitude = (indexLatitude*0.004) + 41.152;
            latitude = Math.rint(latitude*1000)/1000;
            double indexLongitude = Integer.parseInt(e.getActionCommand().substring(posEndLatitude+1));
            double longitude = (indexLongitude*0.004) + (-8.684);
            longitude = Math.rint(longitude*1000)/1000;
            map.setPosition(latitude, longitude);
        }
    }

}
